package Data::Object::Func::Undef;

use Data::Object 'Class';

extends 'Data::Object::Func';

# BUILD
# METHODS

1;

=encoding utf8

=head1 NAME

Data::Object::Func::Undef

=cut

=head1 ABSTRACT

Functions for Undef Data

=cut

=head1 SYNOPSIS

  use Data::Object::Func::Undef;

=cut

=head1 DESCRIPTION

Data::Object::Func::Undef is an abstract base class for function classes in the
Data::Object::Func::Undef space.

=cut
